Package: RSNNS
Maintainer: Christoph Bergmeir <c.bergmeir@decsai.ugr.es>
License: LGPL (>= 2) | file LICENSE
Title: Neural Networks using the Stuttgart Neural Network Simulator
        (SNNS)
LinkingTo: Rcpp
Type: Package
LazyLoad: yes
Authors@R: c(
  person("Christoph", "Bergmeir", email = "c.bergmeir@decsai.ugr.es", role = c("aut", "cre", "cph")),
  person("José M.", "Benítez", role = c("ths")),
  person("Andreas", "Zell", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Niels", "Mache", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Günter", "Mamier", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Michael", "Vogt", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Sven", "Döring", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Ralf", "Hübner", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Kai-Uwe", "Herrmann", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Tobias", "Soyez", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Michael", "Schmalzl", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Tilman", "Sommer", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Artemis", "Hatzigeorgiou", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Dietmar", "Posselt", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Tobias", "Schreiner", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Bernward", "Kett", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Martin", "Reczko", role = c("ctb"), comment= "Part of original SNNS external contributors"),
  person("Martin", "Riedmiller", role = c("ctb"), comment= "Part of original SNNS external contributors"),
  person("Mark", "Seemann", role = c("ctb"), comment= "Part of original SNNS external contributors"),
  person("Marcus", "Ritt", role = c("ctb"), comment= "Part of original SNNS external contributors"),
  person("Jamie", "DeCoster", role = c("ctb"), comment= "Part of original SNNS external contributors"),
  person("Jochen", "Biedermann", role = c("ctb"), comment= "Part of original SNNS external contributors"),
  person("Joachim", "Danz", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Christian", "Wehrfritz", role = c("ctb"), comment= "Part of original SNNS development team"),
  person("Patrick", "Kursawe", role = c("ctb"), comment= "Contributors to SNNS Version 4.3"),
  person("Andre", "El-Ama", role = c("ctb"), comment= "Contributors to SNNS Version 4.3"))
Copyright: Original SNNS software Copyright (C) 1990-1995 SNNS Group,
        IPVR, Univ. Stuttgart, FRG; 1996-1998 SNNS Group, WSI, Univ.
        Tuebingen, FRG. R interface Copyright (C) DiCITS Lab, Sci2s
        group, DECSAI, University of Granada.
Description: The Stuttgart Neural Network Simulator (SNNS) is a library
    containing many standard implementations of neural networks. This
    package wraps the SNNS functionality to make it available from
    within R. Using the 'RSNNS' low-level interface, all of the
    algorithmic functionality and flexibility of SNNS can be accessed.
    Furthermore, the package contains a convenient high-level
    interface, so that the most common neural network topologies and
    learning algorithms integrate seamlessly into R.
Version: 0.4-18
URL: https://github.com/cbergmeir/RSNNS
BugReports: https://github.com/cbergmeir/RSNNS/issues
MailingList: rsnns@googlegroups.com
Date: 2026-01-30
Depends: R (>= 2.10.0), methods, Rcpp (>= 0.8.5)
Suggests: scatterplot3d,NeuralNetTools,plot3D
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-01-29 22:32:24 UTC; bergmeir
Author: Christoph Bergmeir [aut, cre, cph],
  José M. Benítez [ths],
  Andreas Zell [ctb] (Part of original SNNS development team),
  Niels Mache [ctb] (Part of original SNNS development team),
  Günter Mamier [ctb] (Part of original SNNS development team),
  Michael Vogt [ctb] (Part of original SNNS development team),
  Sven Döring [ctb] (Part of original SNNS development team),
  Ralf Hübner [ctb] (Part of original SNNS development team),
  Kai-Uwe Herrmann [ctb] (Part of original SNNS development team),
  Tobias Soyez [ctb] (Part of original SNNS development team),
  Michael Schmalzl [ctb] (Part of original SNNS development team),
  Tilman Sommer [ctb] (Part of original SNNS development team),
  Artemis Hatzigeorgiou [ctb] (Part of original SNNS development team),
  Dietmar Posselt [ctb] (Part of original SNNS development team),
  Tobias Schreiner [ctb] (Part of original SNNS development team),
  Bernward Kett [ctb] (Part of original SNNS development team),
  Martin Reczko [ctb] (Part of original SNNS external contributors),
  Martin Riedmiller [ctb] (Part of original SNNS external contributors),
  Mark Seemann [ctb] (Part of original SNNS external contributors),
  Marcus Ritt [ctb] (Part of original SNNS external contributors),
  Jamie DeCoster [ctb] (Part of original SNNS external contributors),
  Jochen Biedermann [ctb] (Part of original SNNS external contributors),
  Joachim Danz [ctb] (Part of original SNNS development team),
  Christian Wehrfritz [ctb] (Part of original SNNS development team),
  Patrick Kursawe [ctb] (Contributors to SNNS Version 4.3),
  Andre El-Ama [ctb] (Contributors to SNNS Version 4.3)
Repository: CRAN
Date/Publication: 2026-01-30 06:11:03 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 16:55:40 UTC; unix
Archs: RSNNS.so.dSYM
