# Generated by roxygen2: do not edit by hand

S3method(art1,default)
S3method(art2,default)
S3method(artmap,default)
S3method(assoz,default)
S3method(dlvq,default)
S3method(elman,default)
S3method(jordan,default)
S3method(mlp,default)
S3method(plotIterativeError,rsnns)
S3method(predict,rsnns)
S3method(print,rsnns)
S3method(rbf,default)
S3method(rbfDDA,default)
S3method(som,default)
S3method(summary,rsnns)
S3method(train,rsnns)
S3method(weightMatrix,rsnns)
export(SnnsRObjectFactory)
export(analyzeClassification)
export(art1)
export(art2)
export(artmap)
export(assoz)
export(confusionMatrix)
export(decodeClassLabels)
export(denormalizeData)
export(dlvq)
export(elman)
export(encodeClassLabels)
export(exportToSnnsNetFile)
export(extractNetInfo)
export(getNormParameters)
export(getSnnsRDefine)
export(getSnnsRFunctionTable)
export(inputColumns)
export(jordan)
export(matrixToActMapList)
export(mlp)
export(normTrainingAndTestSet)
export(normalizeData)
export(outputColumns)
export(plotActMap)
export(plotIterativeError)
export(plotROC)
export(plotRegressionError)
export(rbf)
export(rbfDDA)
export(readPatFile)
export(readResFile)
export(resolveSnnsRDefine)
export(rsnnsObjectFactory)
export(savePatFile)
export(setSnnsRSeedValue)
export(som)
export(splitForTrainingAndTest)
export(toNumericClassLabels)
export(train)
export(vectorToActMap)
export(weightMatrix)
import(Rcpp)
import(methods)
importFrom(graphics,abline)
importFrom(graphics,image)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(stats,fitted)
importFrom(stats,lm)
importFrom(stats,sd)
useDynLib(RSNNS, .registration=TRUE)
